@@ -44,72 +44,97 @@ class _XkbOption(Gtk.Expander, Tweak):
         vbox.set_margin_start(15)
         self.add(vbox)
 
+        self._multiple_selection = not group_id in { 'keypad', 'kpdl', 'caps', 'altwin', 'nbsp', 'esperanto' }
         self._group_id = group_id
         self._parent_settings = parent_settings
         self._xkb_info = xkb_info
-        self._value = None
         self._possible_values = []
 
-        model_values = [(None, _("Disabled"))]
+        model_values = []
+        if not self._multiple_selection:
+            model_values.append((None, _("Disabled")))
+
         for option_id in self._xkb_info.get_options_for_group(group_id):
             desc = self._xkb_info.description_for_option(group_id, option_id)
             model_values.append((option_id, desc))
             self._possible_values.append(option_id)
 
-        self._radios = dict()
+        def values_cmp((av, ad), (bv, bd)):
+            if not av:
+                return -1
+            elif not bv:
+                return 1
+            else:
+                return cmp(ad, bd)
+        model_values.sort(cmp=values_cmp)
+
+        self._widgets = dict()
         for (val, name) in model_values:
-            self._radios[val] = r = Gtk.RadioButton.new_from_widget(self._radios.get(None))
-            vbox.add(r)
+            w = None
+            if self._multiple_selection:
+                w = Gtk.CheckButton.new()
+            else:
+                w = Gtk.RadioButton.new_from_widget(self._widgets.get(None))
+            self._widgets[val] = w;
+            vbox.add(w)
             l = Gtk.Label(label=name)
             l.set_line_wrap(True)
-            r.add(l)
-            r._changed_id = r.connect('toggled', self._on_radio_changed)
-            r._val = val
+            w.add(l)
+            w._changed_id = w.connect('toggled', self._on_toggled)
+            w._val = val
 
         self.widget_for_size_group = None
         self.reload()
 
     def reload(self):
+        self._values = []
         for v in self._parent_settings.get_strv(TypingTweakGroup.XKB_GSETTINGS_NAME):
             if (v in self._possible_values):
-                self._value = v
-                self._update_radios()
-                return
+                self._values.append(v)
 
-        self._value = None
-        self._update_radios()
+        self._update_checks()
 
-    def _update_radios(self):
-        if self._value:
+    def _update_checks(self):
+        if len(self._values) > 0:
             self.set_label('<b>'+self.name+'</b>')
             self.set_use_markup(True)
         else:
             self.set_label(self.name)
 
-        r = self._radios.get(self._value)
-        if r:
-            r.disconnect(r._changed_id)
-            r.set_active(True)
-            r._changed_id = r.connect('toggled', self._on_radio_changed)
+        def _set_active(w, active):
+            w.disconnect(w._changed_id)
+            w.set_active(active)
+            w._changed_id = w.connect('toggled', self._on_toggled)
 
-    def _on_radio_changed(self, r):
-        if not r.get_active():
-            return
-
-        if not r._val:
-            if self._value:
-                self._parent_settings.setting_remove_from_list(TypingTweakGroup.XKB_GSETTINGS_NAME, self._value)
+        if not self._multiple_selection:
+            if len(self._values) > 0:
+                w = self._widgets.get(self._values[0])
+                if w:
+                    _set_active(w, True)
         else:
-            if self._value:
-                self._parent_settings.setting_remove_from_list(TypingTweakGroup.XKB_GSETTINGS_NAME, self._value)
-            self._parent_settings.setting_add_to_list(TypingTweakGroup.XKB_GSETTINGS_NAME, r._val)
+            for w in self._widgets.values():
+                if w._val in self._values:
+                    _set_active(w, True)
+                else:
+                    _set_active(w, False)
+
+    def _on_toggled(self, w):
+        active = w.get_active()
+        if not self._multiple_selection and active:
+            for v in self._values:
+                self._parent_settings.setting_remove_from_list(TypingTweakGroup.XKB_GSETTINGS_NAME, v)
+
+        if w._val in self._values and not active:
+            self._parent_settings.setting_remove_from_list(TypingTweakGroup.XKB_GSETTINGS_NAME, w._val)
+        elif active and not w._val in self._values and w._val:
+            self._parent_settings.setting_add_to_list(TypingTweakGroup.XKB_GSETTINGS_NAME, w._val)
 
 class TypingTweakGroup(Gtk.Box, TweakGroup):
 
     XKB_GSETTINGS_SCHEMA = "org.gnome.desktop.input-sources"
     XKB_GSETTINGS_NAME = "xkb-options"
-    # These are configurable in gnome-control-center. grp_led is unsupported
-    XKB_OPTIONS_BLACKLIST = {"lv3","Compose key","grp","grp_led"}
+    # grp_led is unsupported
+    XKB_OPTIONS_BLACKLIST = {"grp_led"}
 
     def __init__(self):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=3)
